/*
 * file CcBrowser.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.CcBrowser
 *
 * (C) Copyright IBM Corporation 2006, 2008.  All Rights Reserved
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.stp.client.samples;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.plaf.basic.BasicBorders;
import javax.wvcm.ControllableFolder;
import javax.wvcm.WvcmException;

import com.ibm.rational.stp.client.samples.BrowserDataModel.Operations;
import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.cc.CcActivity;
import com.ibm.rational.wvcm.stp.cc.CcConfigSpec;
import com.ibm.rational.wvcm.stp.cc.CcFile;
import com.ibm.rational.wvcm.stp.cc.CcProvider;
import com.ibm.rational.wvcm.stp.cc.CcResource;
import com.ibm.rational.wvcm.stp.cc.CcStream;
import com.ibm.rational.wvcm.stp.cc.CcView;

/**
 * An extension of the Browser example demonstrating application of ClearCase
 * operations to the displayed resource
 * 
 */
public class CcBrowser implements Operations {

    public CcBrowser(com.ibm.rational.wvcm.stp.StpResource res) {
        m_resource = res;
    }

    /**
     * Returns the operations supported by the type of ClearCase resource being
     * displayed in this window.
     * 
     * @see com.ibm.rational.stp.client.samples.BrowserDataModel.Operations#getList()
     */
    public Vector<String> getList() {
        if (m_resource instanceof CcView)
            return new Vector<String>(Arrays.asList(viewOps));
        else if (m_resource instanceof ControllableFolder)
            return new Vector<String>(Arrays.asList(folderOps));
        else if (m_resource instanceof CcFile)
            return new Vector<String>(Arrays.asList(fileOps));
        else
            return new Vector<String>(Arrays.asList(otherOps));
    }

    /**
     * Supports the operations implemented in the 7.1 CM API
     * <ul>
     * <li>VOB properties
     * <li>Get CQ CqUserDb from Project (Translate CLEARQUEST_USER_DB property
     * from CqUserDbRef to CqUserDb (ref.getUserDb))
     * <li>Create Stream(parent stream, is-read-only, is-integration-stream)
     * <li>Create Activity(headline, comment)
     * <li>Get CQ record for an activity (Translate MY_ACTIVITY_LIST to Record
     * list via CqRecordRef.getRecord().)
     * <li>Create a Web view(new directory name, new tag name) Delete a view
     * </ul>
     */
    public void execute(Object op, BrowserDataModel model) {
        try {
            if (op.equals(EDITCS_OP)) {
                showConfigSpec((CcView) m_resource, true);
            } else if (op.equals(VIEWCS_OP)) {
                showConfigSpec((CcView) m_resource, false);
                return; // No need to refresh
            } else if (op.equals(DELETE_OP)) {
                m_resource.doUnbindAll(null);
            } else if (op.equals(MKVIEW_OP)) {
                showResource("New View", mkView(m_resource));
                return;
            } else if (op.equals(MKACT_OP)) {
                showResource("New Activity", mkActivity(m_resource));
                return;
            } else if (op.equals(MKSTREAM_OP)) {
                showResource("New Stream", mkStream(m_resource));
                return;
            } else {
                CcFile res = (CcFile) m_resource;

                if (op.equals(CHECKOUT_OP)) {
                    res.doCheckout(null, null);
                } else if (op.equals(CHECKIN_OP)) {
                    res.doCheckin(null, null);
                } else if (op.equals(UNCHECKOUT_OP)) {
                    res.doUncheckout(null);
                } else if (op.equals(CONTROL_OP)) {
                    res.doVersionControl(null);
                } else if (op.equals(LOAD_OP)) {
                    res.doLoad(null);
                } else if (op.equals(REFRESH_OP)) {
                    res.doRefresh(null, null);
                } else if (op.equals(EDIT_OP)) {
                    File file = ((StpLocation) res.location()).getFile();
                    Runtime.getRuntime().exec("Notepad.exe \""
                        + file.getCanonicalFile() + "\"");
                    return; // no need to redisplay
                } else if (op.equals(MKFILE_OP)) {
                    showResource("New File", mkFile(m_resource));
                    return; // no need to redisplay
                } else if (op.equals(MKDIR_OP)) {
                    showResource("New Directory", mkDir(m_resource));
                    return; // no need to redisplay
                } else
                    return;
            }

            model.redisplay();
        } catch (Throwable t) {
            Utilities.exception(null, "Invalid resource", t);
        }
    }

    /**
     * Displays the Config Spec for a specified ClearCase View and, optionally,
     * allows the user to edit it.
     * 
     * @param view
     *            The Workspace proxy for the ClearCase view whose ConfigSpec is
     *            to be displayed. Must not be null and must define the
     *            CONFIG_SPEC property.
     * @param allowEdit
     *            If true, the displayed Config Spec is made editable and a
     *            button is added to the window for the user to click to commit
     *            the modified Config Spec.
     * @throws WvcmException
     *             thrown if the required properties are not defined by the
     *             given proxy.
     */
    private void showConfigSpec(final CcView view, boolean allowEdit)
                    throws WvcmException {
        final CcConfigSpec cs = view.getConfigSpec();
        final JTextArea text = new JTextArea(680, 400);

        text.append(cs.getElementRules());
        text.append(BAR);
        text.append(cs.getLoadRules());
        text.setBorder(BasicBorders.getTextFieldBorder());
        text.setCaretPosition(0);
        text.setFont(new Font("Monospaced", Font.PLAIN, 12));

        JFrame frame = new JFrame("Config Spec for " + view.getDisplayName());
        final JDialog dialog = new JDialog(frame);
        final JPanel panel = new JPanel(new BorderLayout());

        if (allowEdit) {
            final JPanel subpanel = new JPanel(new FlowLayout());
            JButton update = new JButton("Update");

            subpanel.add(update);
            update.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent arg0) {
                    // set the return value here
                    try {
                        String newSpec = text.getText();
                        int sep = newSpec.indexOf(BAR);

                        cs.setElementRules(newSpec.substring(0, sep));
                        cs.setLoadRules(newSpec.substring(sep + BAR.length()));

                        view.setConfigSpec(cs);
                        view.doWriteProperties(null);
                        dialog.setVisible(false);
                    } catch (Throwable t) {
                        Utilities.exception(dialog, "Update Config Spec", t);
                    }
                }
            });

            panel.add(subpanel, BorderLayout.SOUTH);

            dialog.setModal(true);
        }

        panel.add(new JScrollPane(text), BorderLayout.CENTER);

        dialog.setContentPane(panel);
        dialog.setBounds(200, 300, 600, 300);
        dialog.setVisible(true);
        dialog.setVisible(false);
    }

    /**
     * A convenience method for invoking {@link Browser#showResource 
     * Browser.showResource} on a given resource only if it is not null.
     * 
     * @param name A brief description of the resource type for inclusion in
     * the title of window that will be used to display the resource
     * @param res The resource to be displayed. May be null.
     * @throws WvcmException If there are problems viewing a non-null resource.
     */
    void showResource(String name, CcResource res) throws WvcmException {
        if (res != null) {
            Browser.showResource(name + " " + res.location(),
                                 res,
                                 Browser.META_PROPS.toArray(),
                                 true);
        }
    }

    /**
     * Creates a new file after getting its name from the user. 
     * @param dir The directory in which to create the new file.
     * @return A ControllableResource proxy representing the newly created
     * file.
     */
    CcFile mkFile(javax.wvcm.Resource dir) {
        try {
            String[] names = { "New File Location" };
            // Todo Need to decode the selector name
            String[] args = { ((StpLocation) dir.location()).getCanonicalPath() };

            getArgs(names, args);

            File file = new File(args[0]);
            CcProvider provider = (CcProvider) dir.provider();
            StpLocation fileLoc = provider
                .filePathLocation(StpProvider.Domain.CLEAR_CASE, file);

            file.createNewFile();
            // TODO Need to modify this for running on UNIX systems
            Runtime.getRuntime().exec("Notepad.exe \""
                + file.getCanonicalFile() + "\"");

            return provider.ccFile(fileLoc);
        } catch (Throwable t) {
            Utilities.exception(null, "Make File", t);
        }

        return null;
    }

    /**
     * Creates a new directory after obtaining its name from the user
     * @param parent The default context in which the directory will be 
     * constructed. May be changed by user.
     * @return A ControllableFolder for the newly constructed directory.
     */
    CcFile mkDir(javax.wvcm.Resource parent) {
        try {
            String[] names = { "New Directory Location" };
            String[] args =
                { ((StpLocation) parent.location()).getCanonicalPath() };

            getArgs(names, args);

            File dir = new File(args[0]);
            CcProvider provider = (CcProvider) parent.provider();
            StpLocation fileLoc = provider
                .filePathLocation(StpProvider.Domain.CLEAR_CASE, dir);

            dir.mkdir();

            return provider.ccDirectory(fileLoc);
        } catch (Throwable t) {
            Utilities.exception(null, "Make Directory", t);
        }

        return null;
    }

    /**
     * Creates a ClearCase view after getting creation parameters from the user.
     * @param context The Default context in which the view is to be created.
     * May be modified by the user.
     * @return A Workspace proxy for the created view. Will be null if the
     * view was not created.
     */
    CcView mkView(javax.wvcm.Resource context) {
        try {
            String[] names = { "View Tag", "File Area", "Comment" };
            String[] args = { null, context.location().string(), "Create view" };

            getArgs(names, args);

            CcProvider provider = (CcProvider) context.provider();
            StpLocation clientLoc = (StpLocation) provider.location(args[1]);
            CcView clientWs = provider.ccView(clientLoc);

            clientWs.setComment(args[2]);
            // TODO When implemented...
            // clientWs.setIsUcmView(args[3]);
            //
            // if (clientWs.getIsUcmView()) {
            // clientWs.setProject(args[4]);
            // clientWs.setStream(args[5]);
            // }
            clientWs.doCreateResource(null);
        } catch (Throwable t) {
            Utilities.exception(null, "Make View", t);
        }
        
        return null;
    }

    /**
     * Creates a new UCM Stream after obtaining parameters from user.
     * @param context The default context in which to create the new UCM Stream.
     * May be modified by the user.
     * @return A UcmStream proxy for the new stream; null if the stream was not
     * created.
     */
    CcStream mkStream(javax.wvcm.Resource context) {
        try {
            String[] args = getArgs(new String[] { "Name (required)",
                "Parent Stream/Project (required)",
                "Is Read Only (true/false)",
                "Default Deliver Target (optional)" });
            CcProvider provider = (CcProvider) context.provider();
            StpLocation streamLoc = (StpLocation) provider.location(args[0]);
            CcStream stream = provider.ccStream(streamLoc);
            StpLocation parentSelector = provider.stpLocation(args[1]);

            if (parentSelector.getNamespace().equals(StpLocation.Namespace.STREAM)) {
                stream.setIsIntegrationStream(false);
                stream.setParentStream(provider.ccStream((StpLocation) provider
                    .location(args[1])));
            } else {
                stream.setIsIntegrationStream(true);
                stream.setParentProject(provider.ccProject((StpLocation) provider
                    .location(args[1])));
            }

            stream.setIsReadOnly(args[2].startsWith("t"));

            if (args[3].length() > 0) {
                stream.setDefaultDeliverTarget(provider
                    .ccStream((StpLocation) provider.location(args[3])));
            }

            stream.doCreateResource(null);

            return stream;
        } catch (Throwable t) {
            Utilities.exception(null, "Make Stream", t);
        }
        return null;
    }

    /**
     * Creates an activity based on input from the user.
     * @param context The context in which the activity will be created, which
     * may be ignored by the user.
     * @return A UcmActivity proxy for the newly created activity; null if the
     * activity was not created.
     */
    CcActivity mkActivity(javax.wvcm.Resource context) {
        try {
            String[] args = getArgs(new String[] { "Stream (required)",
                "Headline (optional)", "Comment (optional)" });
            CcProvider provider = (CcProvider) context.provider();
            StpLocation streamLoc = (StpLocation) provider.location(args[0]);
            CcStream stream = provider.ccStream(streamLoc);
            StpLocation actloc = (StpLocation) provider.location("cc.activity:foo@"
                + streamLoc.getRepo());
            CcActivity act = provider.ccActivity(actloc);

            act.setStream(stream);
            act.setHeadline(args[1]);
            act.setComment(args[2]);
            act.doCreateResource(null);

            return act;
        } catch (Throwable t) {
            Utilities.exception(null, "Make Activity", t);
        }

        return null;
    }

    /**
     * Displays a list of parameters to the user with default values and then
     * allows the user to add or modify values.
     * @param names The names of the parameters to be obtained from the user.
     * @param result On input, the default values to display in the dialog;
     * on output, contains the values entered by the user.
     */
    void getArgs(final String[] names, final String[] result) {
        JFrame frame = new JFrame("Enter Parameters");
        final JDialog dialog = new JDialog(frame);
        final JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        final JTextField[] fields = new JTextField[names.length];

        for (int i = 0; i < names.length; ++i) {
            JTextField field = new JTextField();

            if (result[i] != null)
                field.setText(result[i]);

            fields[i] = field;
            panel.add(new Label(names[i]));
            panel.add(field);
        }

        JButton update = new JButton("Ok");

        panel.add(update);
        update.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                // set the return value here
                try {
                    for (int j = 0; j < names.length; ++j)
                        result[j] = fields[j].getText();

                    dialog.dispose();
                } catch (Throwable t) {
                    Utilities.exception(dialog, "Specify Parameters", t);
                }
            }
        });

        dialog.setModal(true);
        dialog.setContentPane(panel);
        dialog.setBounds(200, 300, 600, 20 + 60 * names.length);
        dialog.setVisible(false);
    }

    /**
     * Obtains a list of parameter values from the user.
     * @param names The names of the parameters that user is supposed to provide.
     * @return A String array contain the user's input for each parameter.
     */
    String[] getArgs(String[] names) {
        final String[] result = new String[names.length];
        getArgs(names, result);

        return result;
    }

    /** 
     * The line of text used to separate the Element Rules from the Load Rules
     * in a Config Spec.
     */
    private static final String BAR = "=== Element Rules/Load Rules Boundary DO NOT REMOVE ===\n";

    final private static String CHECKOUT_OP = "Check Out";

    final private static String CHECKIN_OP = "Check In";

    final private static String UNCHECKOUT_OP = "Uncheck Out";

    final private static String CONTROL_OP = "Control";

    final private static String REFRESH_OP = "Refresh";

    final private static String LOAD_OP = "Load";

    // final private static String UNLOAD_OP = "Unload";
    final private static String EDIT_OP = "Edit";

    final private static String DELETE_OP = "Delete";

    final private static String VIEWCS_OP = "View Config Spec";

    final private static String EDITCS_OP = "Edit Config Spec";

    final private static String MKDIR_OP = "Make Directory";

    final private static String MKFILE_OP = "Make File";

    final private static String MKVIEW_OP = "Make View";

    final private static String MKACT_OP = "Make Activity";

    final private static String MKSTREAM_OP = "Make Stream";

    /** The operations supported on Folders */
    private static final String[] folderOps = { CHECKOUT_OP, CHECKIN_OP,
        UNCHECKOUT_OP, CONTROL_OP, REFRESH_OP, LOAD_OP,
        // UNLOAD_OP,
        DELETE_OP, MKDIR_OP, MKFILE_OP, MKACT_OP, MKSTREAM_OP, MKVIEW_OP };

    /** The operations supported on Files */
    private static final String[] fileOps = { CHECKOUT_OP, CHECKIN_OP,
        UNCHECKOUT_OP, CONTROL_OP, REFRESH_OP, LOAD_OP,
        // UNLOAD_OP,
        EDIT_OP, DELETE_OP, MKACT_OP, MKSTREAM_OP, MKVIEW_OP };

    /** The operations supported on views */
    private static final String[] viewOps = { VIEWCS_OP, EDITCS_OP, REFRESH_OP,
        DELETE_OP, MKDIR_OP, MKACT_OP, MKSTREAM_OP, MKVIEW_OP, };

    /** The operations supported on all resources */
    private static final String[] otherOps = { DELETE_OP, MKACT_OP,
        MKSTREAM_OP, MKVIEW_OP, };

    /** A proxy for the resource being displayed by this Browser window */
    com.ibm.rational.wvcm.stp.StpResource m_resource;

    /**
     * Starts the ClearCase version of the Browser using the CcBrower class as
     * the source for resource operations.
     * 
     * @param args
     *            Not used in this application
     */
    public static void main(String[] args) throws Exception {
        Browser.g_operationsClass = CcBrowser.class;

        Browser.main(args);
    }

}
